/* 
 * Struck: Structured Output Tracking with Kernels
 * 
 * Code to accompany the paper:
 *   Struck: Structured Output Tracking with Kernels
 *   Sam Hare, Amir Saffari, Philip H. S. Torr
 *   International Conference on Computer Vision (ICCV), 2011
 * 
 * Copyright (C) 2011 Sam Hare, Oxford Brookes University, Oxford, UK
 * 
 * This file is part of Struck.
 * 
 * Struck is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Struck is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Struck.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "HaarFeatures.h"
#include "Config.h"

static const int kSystematicFeatureCount = 192;

HaarFeatures::HaarFeatures(const Config& conf):m_config(conf)
{
	//SetCount(kSystematicFeatureCount);
    switch(conf.colorKind){
	case Config::kIntensity:
		SetCount(kSystematicFeatureCount);
		break;
	case Config::kOPPONENT:
		SetCount(kSystematicFeatureCount*3);
		break;
	case Config::kChromatic:
		SetCount(kSystematicFeatureCount*2);
		break;
	case Config::kNChromatic:
		SetCount(kSystematicFeatureCount*2);
		break;
	case Config::kHue:
		SetCount(kSystematicFeatureCount);
		break;
	case Config::kRGB:
		SetCount(kSystematicFeatureCount*3);
		break;
	case Config::kNRGB:
		SetCount(kSystematicFeatureCount*2);
		break;
	case Config::kHSV:
		SetCount(kSystematicFeatureCount*3);
		break;
	case Config::kTRGB:
		SetCount(kSystematicFeatureCount*3);
		break;
	case Config::kLAB:
		SetCount(kSystematicFeatureCount*3);
		break;
	case Config::kYCbCr:
		SetCount(kSystematicFeatureCount*3);
		break;
	default:
		break;
	}

	GenerateSystematic();
}

void HaarFeatures::GenerateSystematic()
{
	float x[] = {0.2f, 0.4f, 0.6f, 0.8f};
	float y[] = {0.2f, 0.4f, 0.6f, 0.8f};
	float s[] = {0.2f, 0.4f};
	for (int iy = 0; iy < 4; ++iy)
	{
		for (int ix = 0; ix < 4; ++ix)
		{
			for (int is = 0; is < 2; ++is)
			{
				FloatRect r(x[ix]-s[is]/2, y[iy]-s[is]/2, s[is], s[is]);
				for (int it = 0; it < 6; ++it)
				{
					m_features.push_back(HaarFeature(r, it));
				}
			}
		}
	}
}

void HaarFeatures::UpdateFeatureVector(const Sample& s)
{
	/*for (int i = 0; i < m_featureCount; ++i)
	{
		m_featVec[i] = m_features[i].Eval(s);
	}*/
	
	switch(m_config.colorKind){
	case Config::kIntensity:
		for (int i = 0; i < m_featureCount; ++i)
		{
			m_featVec[i] = m_features[i].Eval(s);
		}
		break;
	case Config::kOPPONENT:
		for(int j=0;j<3;j++){
			for (int i = 0; i < m_featureCount/3; ++i)
			{	
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kChromatic:
		for(int j=0;j<2;j++){
			for (int i = 0; i < m_featureCount/2; ++i)
			{	
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kNChromatic:
		for(int j=0;j<2;j++){
			for (int i = 0; i < m_featureCount/2; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kHue:
		for(int j=0;j<1;j++){
			for (int i = 0; i < m_featureCount; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kRGB:
		for(int j=0;j<3;j++){
			for (int i = 0; i < m_featureCount/3; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kNRGB:
		for(int j=0;j<2;j++){
			for (int i = 0; i < m_featureCount/2; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kTRGB:
		for(int j=0;j<3;j++){
			for (int i = 0; i < m_featureCount/3; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kHSV:
		for(int j=0;j<3;j++){
			for (int i = 0; i < m_featureCount/3; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kLAB:
		for(int j=0;j<3;j++){
			for (int i = 0; i < m_featureCount/3; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	case Config::kYCbCr:
		for(int j=0;j<3;j++){
			for (int i = 0; i < m_featureCount/3; ++i)
			{
				m_featVec[j*kSystematicFeatureCount+i] = m_features[i].Eval(s,j);
			}
		}
		break;
	default:
		break;
	}
	
}
