/* 
 * Struck: Structured Output Tracking with Kernels
 * 
 * Code to accompany the paper:
 *   Struck: Structured Output Tracking with Kernels
 *   Sam Hare, Amir Saffari, Philip H. S. Torr
 *   International Conference on Computer Vision (ICCV), 2011
 * 
 * Copyright (C) 2011 Sam Hare, Oxford Brookes University, Oxford, UK
 * 
 * This file is part of Struck.
 * 
 * Struck is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Struck is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Struck.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "ImageRep.h"

#include <cassert>

#include <opencv/highgui.h>


#include "util.h"
#include <stdio.h>

using namespace std;
using namespace cv;

static const int kNumBins = 16;



ImageRep::ImageRep(const Config &conf, const Mat& image, bool computeIntegral, bool computeIntegralHist, bool colour):
	m_channels(colour ? 3 : 1),
	m_rect(0, 0, image.cols, image.rows),m_config(conf)
{	
	for (int i = 0; i < m_channels; ++i)
	{
		m_images.push_back(Mat(image.rows, image.cols, CV_8UC1));
		if (computeIntegral) m_integralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32SC1));
		if (computeIntegralHist)
		{
			for (int j = 0; j < kNumBins; ++j)
			{
				m_integralHistImages.push_back(Mat(image.rows+1, image.cols+1, CV_32SC1));
			}
		}
	}

	if(m_config.colorKind == Config::kIntensity){
		colour = false;
	}
	else{
		colour = true;
	}
		
	if (colour)
	{
		assert(image.channels() == 3);
		split(image, m_images);

//		imwrite("b.jpg",m_images[0]);
//		imwrite("g.jpg",m_images[1]);
//		imwrite("r.jpg",m_images[2]);
		/*m_rgbImages.push_back(m_images[2]);
		m_rgbImages.push_back(m_images[1]);
		m_rgbImages.push_back(m_images[0]);*/

		Mat o1 = Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat o2 = Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat o3 = Mat::zeros(image.rows,image.cols,CV_32FC1);
     	Mat o1o3 = Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat o2o3 = Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat hue = Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat nr=Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat ng=Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat nb=Mat::zeros(image.rows,image.cols,CV_32FC1);
		/*Mat r=Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat g=Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat b=Mat::zeros(image.rows,image.cols,CV_32FC1);*/
		Mat tr=Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat tg=Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat tb=Mat::zeros(image.rows,image.cols,CV_32FC1);


		Mat floatsrc = Mat::zeros(image.rows,image.cols,CV_32FC1);
		Mat floathsv = Mat::zeros(image.rows,image.cols,CV_32FC1);
		std::vector<cv::Mat> floathsvs;
		for(int i=0;i<3;i++)
			floathsvs.push_back(Mat(image.rows, image.cols, CV_32FC1));

		Mat floatlabsrc = Mat::zeros(image.rows,image.cols,CV_8UC1);
		Mat floatlab = Mat::zeros(image.rows,image.cols,CV_8UC1);
		std::vector<cv::Mat> floatlabs;
		for(int i=0;i<3;i++)
			floatlabs.push_back(Mat(image.rows, image.cols, CV_8UC1));

		float min_1,max_1,min_2,max_2,min_3,max_3;
		
		switch(m_config.colorKind){
		case Config::kRGB:
			m_rgbImages.push_back(m_images[2]);
			m_rgbImages.push_back(m_images[1]);
			m_rgbImages.push_back(m_images[0]);
			
			/*FILE * fph, *fps, * fpv;
			fph=fopen("image_R_struck.txt","w+");
			fps=fopen("image_G_struck.txt","w+");
			fpv=fopen("image_B_struck.txt","w+");

			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fph,"%.0f\t",r.at<float>(i,j));
					fprintf(fps,"%.0f\t",g.at<float>(i,j));
					fprintf(fpv,"%.0f\t",b.at<float>(i,j));
				}
				fprintf(fph,"\n");
				fprintf(fps,"\n");
				fprintf(fpv,"\n");
			}
			fclose(fph);
			fclose(fps);
			fclose(fpv);*/
			break;
		case Config::kNRGB:
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					int tp_sum;
					tp_sum=(int)m_images[0].at<uchar>(i,j)+(int)m_images[1].at<uchar>(i,j)+(int)m_images[2].at<uchar>(i,j)+M_EPS;
					nr.at<float>(i,j)=(int)m_images[2].at<uchar>(i,j)/(float)tp_sum;
					ng.at<float>(i,j)=(int)m_images[1].at<uchar>(i,j)/(float)tp_sum;
					nb.at<float>(i,j)=(int)m_images[0].at<uchar>(i,j)/(float)tp_sum;
				}
			}
			min_1=min_2=min_3=M_INF;
			max_1=max_2=max_3=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					float rv = nr.at<float>(i,j);
					float gv = ng.at<float>(i,j);
					float bv=  nb.at<float>(i,j);
					if(rv>max_1){
						max_1=rv;
					}
					if(rv<min_1){
						min_1=rv;
					}
					if(gv>max_2){
						max_2=gv;
					}
					if(gv<min_2){
						min_2=gv;
					}
					if(bv>max_3){
						max_3=bv;
					}
					if(bv<min_3){
						min_3=bv;
					}
					
				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					if(min_1==max_1){
						nr.at<float>(i,j)=0.5;
					}
					else{
						nr.at<float>(i,j)=(nr.at<float>(i,j)-min_1)/(max_1-min_1);
					}
					nr.at<float>(i,j)=nr.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_2==max_2){
						ng.at<float>(i,j)=0.5;
					}
					else{
						ng.at<float>(i,j)=(ng.at<float>(i,j)-min_2)/(max_2-min_2);
					}
					ng.at<float>(i,j)=ng.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_3==max_3){
						nb.at<float>(i,j)=0.5;
					}
					else{
						nb.at<float>(i,j)=(nb.at<float>(i,j)-min_3)/(max_3-min_3);
					}
					nb.at<float>(i,j)=nb.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
				}
			}
			/*FILE * fph, *fps, * fpv;
			fph=fopen("image_NR_struck.txt","w+");
			fps=fopen("image_NG_struck.txt","w+");
			fpv=fopen("image_NB_struck.txt","w+");

			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fph,"%.4f\t",nr.at<float>(i,j));
					fprintf(fps,"%.4f\t",ng.at<float>(i,j));
					fprintf(fpv,"%.4f\t",nb.at<float>(i,j));
				}
				fprintf(fph,"\n");
				fprintf(fps,"\n");
				fprintf(fpv,"\n");
			}
			fclose(fph);
			fclose(fps);
			fclose(fpv);*/
			m_nRGBImages.push_back(nr);
			m_nRGBImages.push_back(ng);
	//		m_nRGBImages.push_back(nb);
			break;
		case Config::kTRGB:
			float tp_sum,tp_mean,tp_std;
			for(int i=0;i<3;i++){
				tp_sum=tp_mean=tp_std=0;
				for(int j=0;j<image.rows;j++){
					for(int k=0;k<image.cols;k++){
						tp_sum+=(int)m_images[i].at<uchar>(j,k);
					}
				}
				tp_mean=tp_sum/(image.rows*image.cols);
				for(int j=0;j<image.rows;j++){
					for(int k=0;k<image.cols;k++){
						float tp_val=(int)m_images[i].at<uchar>(j,k);
						tp_std+=(tp_val-tp_mean)*(tp_val-tp_mean);
					}
				}
				tp_std=tp_std/(image.rows*image.cols-1);
				tp_std=sqrt(tp_std);
				for(int j=0;j<image.rows;j++){
					for(int k=0;k<image.cols;k++){
						float tp_val=(int)m_images[i].at<uchar>(j,k);
						if(i==0){
							if (abs(tp_std)>1e-10){
								tb.at<float>(j,k)=(tp_val-tp_mean)/tp_std;
							//	cout<<tb.at<float>(j,k)<<endl;
							}
							else{
								tb.at<float>(j,k)=0;
							}
						}
						else if(i==1){
							if (abs(tp_std)>1e-10){
								tg.at<float>(j,k)=(tp_val-tp_mean)/tp_std;
							}
							else{
								tg.at<float>(j,k)=0;
							}
						}
						else if(i==2){
							if (abs(tp_std)>1e-10){
								tr.at<float>(j,k)=(tp_val-tp_mean)/tp_std;
							}
							else{
								tr.at<float>(j,k)=0;
							}

						}
						else{
							fprintf(stderr, "%s %d: ERROR: %s\n", __FILE__, __LINE__, "The index is out of range" );
						}
					}
				}
			}
			
			m_tRGBImages.push_back(tr);
			m_tRGBImages.push_back(tg);
			m_tRGBImages.push_back(tb);
			/*FILE * fph, *fps, * fpv;
			fph=fopen("image_TR_struck.txt","w+");
			fps=fopen("image_TG_struck.txt","w+");
			fpv=fopen("image_TB_struck.txt","w+");

			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fph,"%.4f\t",tr.at<float>(i,j));
					fprintf(fps,"%.4f\t",tg.at<float>(i,j));
					fprintf(fpv,"%.4f\t",tb.at<float>(i,j));
				}
				fprintf(fph,"\n");
				fprintf(fps,"\n");
				fprintf(fpv,"\n");
			}
			fclose(fph);
			fclose(fps);
			fclose(fpv);*/
			break;
		case Config::kOPPONENT:
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					//cout<<(int)m_images[2].at<uchar>(i,j)<<"  "<<(int)m_images[1].at<uchar>(i,j)<<endl;
					o1.at<float>(i,j)=((int)m_images[2].at<uchar>(i,j)-(int)m_images[1].at<uchar>(i,j))/sqrt(2.0);
					//cout<<o1.at<float>(i,j)<<endl;
					o2.at<float>(i,j)=((int)m_images[2].at<uchar>(i,j)+(int)m_images[1].at<uchar>(i,j)-2*(int)m_images[0].at<uchar>(i,j))/sqrt(6.0);
					//cout<<o2.at<float>(i,j)<<endl;
					o3.at<float>(i,j)=((int)m_images[0].at<uchar>(i,j)+(int)m_images[1].at<uchar>(i,j)+(int)m_images[2].at<uchar>(i,j))/sqrt(3.0);
					//cout<<o3.at<float>(i,j)<<endl;
				}
			}

			min_1=min_2=min_3=M_INF;
			max_1=max_2=max_3=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					float vo1 = o1.at<float>(i,j);
					float vo2 = o2.at<float>(i,j);
					float vo3=  o3.at<float>(i,j);
					if(vo1>max_1){
						max_1=vo1;
					}
					if(vo1<min_1){
						min_1=vo1;
					}
					if(vo2>max_2){
						max_2=vo2;
					}
					if(vo2<min_2){
						min_2=vo2;
					}
					if(vo3>max_3){
						max_3=vo3;
					}
					if(vo3<min_3){
						min_3=vo3;
					}
					
				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					if(min_1==max_1){
						o1.at<float>(i,j)=0.5;
					}
					else{
						o1.at<float>(i,j)=(o1.at<float>(i,j)-min_1)/(max_1-min_1);
					}
					o1.at<float>(i,j)=o1.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_2==max_2){
						o2.at<float>(i,j)=0.5;
					}
					else{
						o2.at<float>(i,j)=(o2.at<float>(i,j)-min_2)/(max_2-min_2);
					}
					o2.at<float>(i,j)=o2.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_3==max_3){
						o3.at<float>(i,j)=0.5;
					}
					else{
						o3.at<float>(i,j)=(o3.at<float>(i,j)-min_3)/(max_3-min_3);
					}
					o3.at<float>(i,j)=o3.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
				}
			}
			m_oppoImages.push_back(o1);
			m_oppoImages.push_back(o2);
			m_oppoImages.push_back(o3);
			break;
		case Config::kChromatic:
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					//cout<<(int)m_images[2].at<uchar>(i,j)<<"  "<<(int)m_images[1].at<uchar>(i,j)<<endl;
					o1.at<float>(i,j)=((int)m_images[2].at<uchar>(i,j)-(int)m_images[1].at<uchar>(i,j))/sqrt(2.0);
					//cout<<o1.at<float>(i,j)<<endl;
					o2.at<float>(i,j)=((int)m_images[2].at<uchar>(i,j)+(int)m_images[1].at<uchar>(i,j)-2*(int)m_images[0].at<uchar>(i,j))/sqrt(6.0);
					//cout<<o2.at<float>(i,j)<<endl;
					
				}
			}

			min_1=min_2=M_INF;
			max_1=max_2=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					float vo1 = o1.at<float>(i,j);
					float vo2 = o2.at<float>(i,j);
					if(vo1>max_1){
						max_1=vo1;
					}
					if(vo1<min_1){
						min_1=vo1;
					}
					if(vo2>max_2){
						max_2=vo2;
					}
					if(vo2<min_2){
						min_2=vo2;
					}
				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					if(min_1==max_1){
						o1.at<float>(i,j)=0.5;
					}
					else{
						o1.at<float>(i,j)=(o1.at<float>(i,j)-min_1)/(max_1-min_1);
					}
					o1.at<float>(i,j)=o1.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_2==max_2){
						o2.at<float>(i,j)=0.5;
					}
					else{
						o2.at<float>(i,j)=(o2.at<float>(i,j)-min_2)/(max_2-min_2);
					}
					o2.at<float>(i,j)=o2.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;				
				}
			}
			m_chromaticImages.push_back(o1);
			m_chromaticImages.push_back(o2);
			break;
		case Config::kNChromatic:
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					//cout<<(int)m_images[2].at<uchar>(i,j)<<"  "<<(int)m_images[1].at<uchar>(i,j)<<endl;
					int o1v=((int)m_images[2].at<uchar>(i,j)-(int)m_images[1].at<uchar>(i,j))/sqrt(2.0);
					//cout<<o1.at<float>(i,j)<<endl;
					int o2v=((int)m_images[2].at<uchar>(i,j)+(int)m_images[1].at<uchar>(i,j)-2*(int)m_images[0].at<uchar>(i,j))/sqrt(6.0);
					//cout<<o2.at<float>(i,j)<<endl;
					int o3v=((int)m_images[0].at<uchar>(i,j)+(int)m_images[1].at<uchar>(i,j)+(int)m_images[2].at<uchar>(i,j))/sqrt(3.0);
					//cout<<o3.at<float>(i,j)<<endl;
					if(abs(o3v)<=M_EPS){
						o1o3.at<float>(i,j)=0;
						//cout<<o1o3.at<float>(i,j)<<endl;
						o2o3.at<float>(i,j)=0;
					}
					else{
						o1o3.at<float>(i,j)=(float)o1v/(o3v+M_EPS);
						//cout<<o1o3.at<float>(i,j)<<endl;
						o2o3.at<float>(i,j)=(float)o2v/(o3v+M_EPS);
					}
				}
			}
			min_1=min_2=M_INF;
			max_1=max_2=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					float vo13 = o1o3.at<float>(i,j);
					float vo23 = o2o3.at<float>(i,j);
					if(vo13>max_1){
						max_1=vo13;
					}
					if(vo13<min_1){
						min_1=vo13;
					}
					if(vo23>max_2){
						max_2=vo23;
					}
					if(vo23<min_2){
						min_2=vo23;
					}
				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					if(min_1==max_1){
						o1o3.at<float>(i,j)=0.5;
					}
					else{
						o1o3.at<float>(i,j)=(o1o3.at<float>(i,j)-min_1)/(max_1-min_1);
					}
					o1o3.at<float>(i,j)=o1o3.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_2==max_2){
						o2o3.at<float>(i,j)=0.5;
					}
					else{
						o2o3.at<float>(i,j)=(o2o3.at<float>(i,j)-min_2)/(max_2-min_2);
					}
					o2o3.at<float>(i,j)=o2o3.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;				
				}
			}
			m_nChromaticImages.push_back(o1o3);
			m_nChromaticImages.push_back(o2o3);
			break;
		case Config::kHue:
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					//cout<<(int)m_images[2].at<uchar>(i,j)<<"  "<<(int)m_images[1].at<uchar>(i,j)<<endl;
					int vo1=((int)m_images[2].at<uchar>(i,j)-(int)m_images[1].at<uchar>(i,j))/sqrt(2.0);
					//cout<<o1.at<float>(i,j)<<endl;
					int vo2=((int)m_images[2].at<uchar>(i,j)+(int)m_images[1].at<uchar>(i,j)-2*(int)m_images[0].at<uchar>(i,j))/sqrt(6.0);
					//cout<<o2.at<float>(i,j)<<endl;
					
					//cout<<o3.at<float>(i,j)<<endl;
					if(abs(vo2)<=M_EPS){
						hue.at<float>(i,j)=0;
					}
					else{
						hue.at<float>(i,j)=(float)vo1/(vo2+M_EPS);
					}
				}
			}
			/*FILE * fphue;
			fphue=fopen("image_hue1_struck.txt","w+");
			
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fphue,"%.4f\t",hue.at<float>(i,j));
					
				}
				fprintf(fphue,"\n");
				
			}
			fclose(fphue);
			*/
			min_1=M_INF;
			max_1=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					float vhue = hue.at<float>(i,j);
					if(vhue>max_1){
						max_1=vhue;
					}
					if(vhue<min_1){
						min_1=vhue;
					}
				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
				//	cout<<hue.at<float>(i,j)<<endl;
					if(min_1==max_1){
						hue.at<float>(i,j)=0.5;
					}
					else{
						hue.at<float>(i,j)=(hue.at<float>(i,j)-min_1)/(max_1-min_1);
					}
				//	cout<<hue.at<float>(i,j)<<endl;
					hue.at<float>(i,j)=hue.at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;	
				}
			}
			m_hueImages.push_back(hue);
			/*FILE * fphue;
			fphue=fopen("image_hue_struck.txt","w+");
			
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fphue,"%.4f\t",hue.at<float>(i,j));
					
				}
				fprintf(fphue,"\n");
				
			}
			fclose(fphue);
			*/
			break;
		case Config::kHSV:
			image.convertTo(floatsrc,CV_32F,1.0/255.0);
			cvtColor(floatsrc,floathsv, CV_BGR2HSV);
			split(floathsv,floathsvs);

			min_1=min_2=min_3=M_INF;
			max_1=max_2=max_3=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					float hv = floathsvs[0].at<float>(i,j);
					float sv = floathsvs[1].at<float>(i,j);
					float vv=  floathsvs[2].at<float>(i,j);
					if(hv>max_1){
						max_1=hv;
					}
					if(hv<min_1){
						min_1=hv;
					}
					if(sv>max_2){
						max_2=sv;
					}
					if(sv<min_2){
						min_2=sv;
					}
					if(vv>max_3){
						max_3=vv;
					}
					if(vv<min_3){
						min_3=vv;
					}
					
				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					if(min_1==max_1){
						floathsvs[0].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[0].at<float>(i,j)=(floathsvs[0].at<float>(i,j)-min_1)/(max_1-min_1);
					}
					floathsvs[0].at<float>(i,j)=floathsvs[0].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_2==max_2){
						floathsvs[1].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[1].at<float>(i,j)=(floathsvs[1].at<float>(i,j)-min_2)/(max_2-min_2);
					}
					floathsvs[1].at<float>(i,j)=floathsvs[1].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_3==max_3){
						floathsvs[2].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[2].at<float>(i,j)=(floathsvs[2].at<float>(i,j)-min_3)/(max_3-min_3);
					}
					floathsvs[2].at<float>(i,j)=floathsvs[2].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
				}
			}

			m_hsvImages.push_back(floathsvs[0]);
			m_hsvImages.push_back(floathsvs[1]);
			m_hsvImages.push_back(floathsvs[2]);
			break;
		case Config::kLAB:
			//image.convertTo(floatsrc,CV_32F,1.0/255.0);
			image.convertTo(floatlabsrc,CV_8U,1);
			cvtColor(floatlabsrc,floatlab, CV_BGR2Lab);
			split(floatlab,floatlabs);

			
			/*FILE * fph, *fps, * fpv;
			fph=fopen("image_H_struck.txt","w+");
			fps=fopen("image_S_struck.txt","w+");
			fpv=fopen("image_V_struck.txt","w+");

			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fph,"%d\t",floatlabs[0].at<uchar>(i,j));
					fprintf(fps,"%d\t",floatlabs[1].at<uchar>(i,j));
					fprintf(fpv,"%d\t",floatlabs[2].at<uchar>(i,j));
				}
				fprintf(fph,"\n");
				fprintf(fps,"\n");
				fprintf(fpv,"\n");
			}
			fclose(fph);
			fclose(fps);
			fclose(fpv);*/

			min_1=min_2=min_3=M_INF;
			max_1=max_2=max_3=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){

					floathsvs[0].at<float>(i,j)=(int)(floatlabs[0].at<uchar>(i,j));
					floathsvs[1].at<float>(i,j)=(int)(floatlabs[1].at<uchar>(i,j));
					floathsvs[2].at<float>(i,j)=(int)(floatlabs[2].at<uchar>(i,j));

					float hv = floathsvs[0].at<float>(i,j);
					float sv = floathsvs[1].at<float>(i,j);
					float vv=  floathsvs[2].at<float>(i,j);
	//				cout<<(int)floatlabs[0].at<uchar>(i,j)<<" "<<hv<<endl;
	//				cout<<(int)floatlabs[1].at<uchar>(i,j)<<" "<<sv<<endl;
	//				cout<<(int)floatlabs[2].at<uchar>(i,j)<<" "<<vv<<endl;
					if(hv>max_1){
						max_1=hv;
					}
					if(hv<min_1){
						min_1=hv;
					}
					if(sv>max_2){
						max_2=sv;
					}
					if(sv<min_2){
						min_2=sv;
					}
					if(vv>max_3){
						max_3=vv;
					}
					if(vv<min_3){
						min_3=vv;
					}

				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					if(min_1==max_1){
						floathsvs[0].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[0].at<float>(i,j)=(floathsvs[0].at<float>(i,j)-min_1)/(max_1-min_1);
					}
					floathsvs[0].at<float>(i,j)=floathsvs[0].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_2==max_2){
						floathsvs[1].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[1].at<float>(i,j)=(floathsvs[1].at<float>(i,j)-min_2)/(max_2-min_2);
					}
					floathsvs[1].at<float>(i,j)=floathsvs[1].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_3==max_3){
						floathsvs[2].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[2].at<float>(i,j)=(floathsvs[2].at<float>(i,j)-min_3)/(max_3-min_3);
					}
					floathsvs[2].at<float>(i,j)=floathsvs[2].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
				}
			}

			m_LABImages.push_back(floathsvs[0]);
			m_LABImages.push_back(floathsvs[1]);
			m_LABImages.push_back(floathsvs[2]);
			break;
		case Config::kYCbCr:
			image.convertTo(floatlabsrc,CV_8U,1);
			cvtColor(floatlabsrc,floatlab, CV_BGR2Lab);
			split(floatlab,floatlabs);

		/*	
			FILE * fph, *fps, * fpv;
			fph=fopen("image_H_struck.txt","w+");
			fps=fopen("image_S_struck.txt","w+");
			fpv=fopen("image_V_struck.txt","w+");

			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fph,"%d\t",floatlabs[0].at<uchar>(i,j));
					fprintf(fps,"%d\t",floatlabs[1].at<uchar>(i,j));
					fprintf(fpv,"%d\t",floatlabs[2].at<uchar>(i,j));
				}
				fprintf(fph,"\n");
				fprintf(fps,"\n");
				fprintf(fpv,"\n");
			}
			fclose(fph);
			fclose(fps);
			fclose(fpv);*/

			min_1=min_2=min_3=M_INF;
			max_1=max_2=max_3=M_NINF;
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){

					floathsvs[0].at<float>(i,j)=(int)(floatlabs[0].at<uchar>(i,j));
					floathsvs[1].at<float>(i,j)=(int)(floatlabs[1].at<uchar>(i,j));
					floathsvs[2].at<float>(i,j)=(int)(floatlabs[2].at<uchar>(i,j));

					float hv = floathsvs[0].at<float>(i,j);
					float sv = floathsvs[1].at<float>(i,j);
					float vv=  floathsvs[2].at<float>(i,j);
			//		cout<<(int)floatlabs[0].at<uchar>(i,j)<<" "<<hv<<endl;
			//		cout<<(int)floatlabs[1].at<uchar>(i,j)<<" "<<sv<<endl;
			//		cout<<(int)floatlabs[2].at<uchar>(i,j)<<" "<<vv<<endl;
					if(hv>max_1){
						max_1=hv;
					}
					if(hv<min_1){
						min_1=hv;
					}
					if(sv>max_2){
						max_2=sv;
					}
					if(sv<min_2){
						min_2=sv;
					}
					if(vv>max_3){
						max_3=vv;
					}
					if(vv<min_3){
						min_3=vv;
					}

				}
			}
			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					if(min_1==max_1){
						floathsvs[0].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[0].at<float>(i,j)=(floathsvs[0].at<float>(i,j)-min_1)/(max_1-min_1);
					}
					floathsvs[0].at<float>(i,j)=floathsvs[0].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_2==max_2){
						floathsvs[1].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[1].at<float>(i,j)=(floathsvs[1].at<float>(i,j)-min_2)/(max_2-min_2);
					}
					floathsvs[1].at<float>(i,j)=floathsvs[1].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
					if(min_3==max_3){
						floathsvs[2].at<float>(i,j)=0.5;
					}
					else{
						floathsvs[2].at<float>(i,j)=(floathsvs[2].at<float>(i,j)-min_3)/(max_3-min_3);
					}
					floathsvs[2].at<float>(i,j)=floathsvs[2].at<float>(i,j)*(M_NMAX-M_NMIN)+M_NMIN;
				}
			}

			m_YCbCrImages.push_back(floathsvs[0]);
			m_YCbCrImages.push_back(floathsvs[1]);
			m_YCbCrImages.push_back(floathsvs[2]);
			break;


			/*FILE * fph, *fps, * fpv;
			fph=fopen("image_H_struck.txt","w+");
			fps=fopen("image_S_struck.txt","w+");
			fpv=fopen("image_V_struck.txt","w+");

			for(int i=0;i<image.rows;i++){
				for(int j=0;j<image.cols;j++){
					fprintf(fph,"%.4f\t",floathsvs[0].at<float>(i,j));
					fprintf(fps,"%.4f\t",floathsvs[1].at<float>(i,j));
					fprintf(fpv,"%.4f\t",floathsvs[2].at<float>(i,j));
				}
				fprintf(fph,"\n");
				fprintf(fps,"\n");
				fprintf(fpv,"\n");
			}
			fclose(fph);
			fclose(fps);
			fclose(fpv);*/
		
		default:
			break;
		}


	}
	else
	{
		assert(image.channels() == 1 || image.channels() == 3);
		if (image.channels() == 3)
		{
			cvtColor(image, m_images[0], CV_RGB2GRAY);
		}
		else if (image.channels() == 1)
		{
			image.copyTo(m_images[0]);
		}
	}
	
	if (computeIntegral)
	{
		//for (int i = 0; i < m_channels; ++i)
		//{
		//	//equalizeHist(m_images[i], m_images[i]);
		//	integral(m_images[i], m_integralImages[i]);
		//}
		int tp_size;
		switch(m_config.colorKind){
		case Config::kIntensity:
			for(int i=0;i<1;++i){
				integral(m_images[i],m_integralImages[i]);
			}
			break;
		case Config::kOPPONENT:
			tp_size = m_oppoImages.size();
			for(int i=0;i<tp_size;++i){
				m_oppoIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_oppoImages[i],m_oppoIntegralImages[i],CV_32F);
			}
			break;
		case Config::kChromatic:
			tp_size = m_chromaticImages.size();
			for(int i=0;i<tp_size;++i){
				m_chromaticIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_chromaticImages[i],m_chromaticIntegralImages[i],CV_32F);
			}
			break;
		case Config::kNChromatic:
			tp_size = m_nChromaticImages.size();
			for(int i=0;i<tp_size;++i){
				m_nChromaticIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_nChromaticImages[i],m_nChromaticIntegralImages[i],CV_32F);
			}
			break;
		case Config::kHue:
			tp_size =  m_hueImages.size();
			for(int i=0;i<tp_size;++i){
				m_hueIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_hueImages[i],m_hueIntegralImages[i],CV_32F);
			}
			break;
		case Config::kRGB:
			tp_size =  m_rgbImages.size();
			for(int i=0;i<tp_size;++i){
				m_rgbIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_rgbImages[i],m_rgbIntegralImages[i],CV_32F);
			}
			break;
		case Config::kNRGB:
			tp_size =  m_nRGBImages.size();
			for(int i=0;i<tp_size;++i){
				m_nRGBIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_nRGBImages[i],m_nRGBIntegralImages[i],CV_32F);
			}
			break;
		case Config::kTRGB:
			tp_size =  m_tRGBImages.size();
			for(int i=0;i<tp_size;++i){
				m_tRGBIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_tRGBImages[i],m_tRGBIntegralImages[i],CV_32F);
			}
			break;
		case Config::kHSV:
			tp_size =  m_hsvImages.size();
			for(int i=0;i<tp_size;++i){
				m_hsvIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_hsvImages[i],m_hsvIntegralImages[i],CV_32F);
			}
			break;
		case Config::kLAB:
			tp_size =  m_LABImages.size();
			for(int i=0;i<tp_size;++i){
				m_LABIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_LABImages[i],m_LABIntegralImages[i],CV_32F);
			}
			break;
		case Config::kYCbCr:
			tp_size =  m_YCbCrImages.size();
			for(int i=0;i<tp_size;++i){
				m_YCbCrIntegralImages.push_back(Mat(image.rows+1, image.cols+1, CV_32FC1));
				integral(m_YCbCrImages[i],m_YCbCrIntegralImages[i],CV_32F);
			}
			break;
		default:
			break;
		}

	}
	
	if (computeIntegralHist)
	{
		Mat tmp(image.rows, image.cols, CV_8UC1);
		tmp.setTo(0);
		for (int j = 0; j < kNumBins; ++j)
		{
			for (int y = 0; y < image.rows; ++y)
			{
				const uchar* src = m_images[0].ptr(y);
				uchar* dst = tmp.ptr(y);
				for (int x = 0; x < image.cols; ++x)
				{
					int bin = (int)(((float)*src/256)*kNumBins);
					*dst = (bin == j) ? 1 : 0;
					++src;
					++dst;
				}
			}
			
			integral(tmp, m_integralHistImages[j]);			
		}
	}
}

int ImageRep::Sum(const IntRect& rRect, int channel) const
{
	assert(rRect.XMin() >= 0 && rRect.YMin() >= 0 && rRect.XMax() <= m_images[0].cols && rRect.YMax() <= m_images[0].rows);
	/*return m_integralImages[channel].at<int>(rRect.YMin(), rRect.XMin()) +
			m_integralImages[channel].at<int>(rRect.YMax(), rRect.XMax()) -
			m_integralImages[channel].at<int>(rRect.YMax(), rRect.XMin()) -
			m_integralImages[channel].at<int>(rRect.YMin(), rRect.XMax());*/

	switch(m_config.colorKind){
	case Config::kIntensity:
		return m_integralImages[channel].at<int>(rRect.YMin(), rRect.XMin()) +
			m_integralImages[channel].at<int>(rRect.YMax(), rRect.XMax()) -
			m_integralImages[channel].at<int>(rRect.YMax(), rRect.XMin()) -
			m_integralImages[channel].at<int>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kOPPONENT:
		return m_oppoIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_oppoIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_oppoIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_oppoIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kChromatic:
		return m_chromaticIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_chromaticIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_chromaticIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_chromaticIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kNChromatic:
		return m_nChromaticIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_nChromaticIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_nChromaticIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_nChromaticIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kHue:
		return m_hueIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_hueIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_hueIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_hueIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kRGB:
		return m_rgbIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_rgbIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_rgbIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_rgbIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kNRGB:
		return m_nRGBIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_nRGBIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_nRGBIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_nRGBIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kTRGB:
		return m_tRGBIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_tRGBIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_tRGBIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_tRGBIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kHSV:
		return m_hsvIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_hsvIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_hsvIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_hsvIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kLAB:
		return m_LABIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_LABIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_LABIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_LABIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	case Config::kYCbCr:
		return m_YCbCrIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMin()) +
			m_YCbCrIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMax()) -
			m_YCbCrIntegralImages[channel].at<float>(rRect.YMax(), rRect.XMin()) -
			m_YCbCrIntegralImages[channel].at<float>(rRect.YMin(), rRect.XMax());
		break;
	default:
		break;
	}

}

void ImageRep::Hist(const IntRect& rRect, Eigen::VectorXd& h) const
{
	assert(rRect.XMin() >= 0 && rRect.YMin() >= 0 && rRect.XMax() <= m_images[0].cols && rRect.YMax() <= m_images[0].rows);
	int norm = rRect.Area();
	for (int i = 0; i < kNumBins; ++i)
	{
		int sum = m_integralHistImages[i].at<int>(rRect.YMin(), rRect.XMin()) +
			m_integralHistImages[i].at<int>(rRect.YMax(), rRect.XMax()) -
			m_integralHistImages[i].at<int>(rRect.YMax(), rRect.XMin()) -
			m_integralHistImages[i].at<int>(rRect.YMin(), rRect.XMax());
		h[i] = (float)sum/norm;
	}
}
